@@ -128,20 +128,11 @@ public class PDFAnnotation
 	public native void setFlags(int flags);
 	public native String getContents();
 	public native void setContents(String contents);
-	public native Rect getRect();
-	public native void setRect(Rect rect);
-	public native float getBorder();
-	public native void setBorder(float width);
 	public native float[] getColor();
 	public native void setColor(float[] color);
-	public native boolean hasInteriorColor();
-	public native float[] getInteriorColor();
-	public native void setInteriorColor(float[] color);
 	public native float getOpacity();
 	public native void setOpacity(float opacity);
-	public native boolean hasAuthor();
-	public native String getAuthor();
-	public native void setAuthor(String author);
+
 	protected native long getCreationDateNative();
 	protected native void setCreationDate(long time);
 	protected native long getModificationDateNative();
@@ -159,6 +150,18 @@ public class PDFAnnotation
 		setModificationDate(date.getTime());
 	}
 
+	public native boolean hasRect();
+	public native Rect getRect();
+	public native void setRect(Rect rect);
+
+	public native boolean hasInteriorColor();
+	public native float[] getInteriorColor();
+	public native void setInteriorColor(float[] color);
+
+	public native boolean hasAuthor();
+	public native String getAuthor();
+	public native void setAuthor(String author);
+
 	public native boolean hasLineEndingStyles();
 	public native int[] getLineEndingStyles();
 	public native void setLineEndingStyles(int startStyle, int endStyle);
@@ -265,10 +268,19 @@ public class PDFAnnotation
 	public native boolean hasIcon();
 	public native String getIcon();
 	public native void setIcon(String icon);
+
 	public native boolean hasOpen();
-	public native boolean isOpen();
+	public native boolean getIsOpen();
 	public native void setIsOpen(boolean open);
 
+	public native boolean hasLine();
+	public native Point[] getLine();
+	public native void setLine(Point a, Point b);
+
+	public native boolean hasFileSpecification();
+	public native void setFileSpecification(PDFObject fs);
+	public native PDFObject getFileSpecification();
+
 	public native void eventEnter();
 	public native void eventExit();
 	public native void eventDown();
@@ -284,13 +296,10 @@ public class PDFAnnotation
 	public native int getLanguage();
 	public native void setLanguage(int lang);
 
+	public native boolean hasQuadding();
 	public native int getQuadding();
 	public native void setQuadding(int quadding);
 
-	public native boolean hasLine();
-	public native Point[] getLine();
-	public native void setLine(Point a, Point b);
-
 	public native DefaultAppearance getDefaultAppearance();
 	public native void setDefaultAppearance(String font, float size, float[] color);
 
@@ -332,12 +341,19 @@ public class PDFAnnotation
 		setNativeAppearanceImage(image);
 	}
 
-	public native boolean hasFileSpecification();
-	public native void setFileSpecification(PDFObject fs);
-	public native PDFObject getFileSpecification();
-
 	public native boolean getHiddenForEditing();
 	public native void setHiddenForEditing(boolean hidden);
 
-	public native boolean applyRedaction(boolean blackBoxes, int imageMethod);
+	public boolean applyRedaction(boolean blackBoxes, int imageMethod)
+	{
+		return applyRedaction(blackBoxes, imageMethod, 0);
+	}
+
+	public native boolean applyRedaction(boolean blackBoxes, int imageMethod, int lineArt);
+
+	/** @deprecated use getBorderWidth instead. */
+	public float getBorder() { return getBorderWidth(); }
+
+	/** @deprecated use setBorderWidth instead. */
+	public void setBorder(float width) { setBorderWidth(width); }
 }
